/***************************************************************************
*   Copyright (C) 2004-2012 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXENCODER_H
#define BIBTEXENCODER_H

#include <QString>

/**
 * Base class for that convert between different textual representations
 * for non-ASCII characters. Examples for external textual representations
 * are \"a in LaTeX and &auml; in XML.
 * @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
 */
class Encoder
{
public:
    virtual ~Encoder() {
        /* nothing */
    };

    /**
     * Decode from external textual representation to internal (UTF-8) representation.
     * @param text text in external textual representation
     * @return text in internal (UTF-8) representation
     */
    virtual QString decode(const QString &text) const;

    /**
     * Encode from internal (UTF-8) representation to external textual representation.
     * @param text in internal (UTF-8) representation
     * @return text text in external textual representation
     */
    virtual QString encode(const QString &text) const;
};

#endif
